import os

def createPath(name):
  return os.path.join(os.path.dirname(os.path.abspath(__file__)), "assets", name)

tips_ui = createPath("tips.ui")
tipsBox_ui = createPath("tipsBox.ui")
MainWindow_ui = createPath("main.ui")
loading_icon = createPath("loading.gif")
small_page_icon = createPath("small_page.png")
maximize_page_icon = createPath("maximize_page.png")

# 定义软件当前版本
APP_VERSION = '1.0.0'
APP_TITLE = '小灰妙记'
APP_ICON_PATH = createPath('favicon.png')

IS_DEBUG = False        #生产环境，使用远程服务器

PING_HOST = 'greatnote.cn'

# 默认主题
default_theme = {
  "skin": {
    "current_skin_id": "skin001"
  },
  "skinStyle": [
    {
      "id": "skin001",
      "name": "\u9ed8\u8ba4",
      "header_bg_color": "#6a5acd",
      "main_bg_color": "#f0f0f0",
      "app_bg_image": "./assets/image/skin/1703964016891.png",
      "current": "skin001"
    },
    {
      "id": "skin002",
      "name": "\u73ca\u745a\u7ea2",
      "header_bg_color": "#cd5a5a",
      "main_bg_color": "#fffbcb",
      "app_bg_image": "./assets/image/skin/1703951087872.jpg",
      "current": "skin001"
    },
    {
      "id": "skin003",
      "name": "\u70ab\u9177\u9ed1",
      "header_bg_color": "#000",
      "main_bg_color": "#e4d5ff",
      "app_bg_image": "./assets/image/skin/1703951426046.jpg",
      "current": "skin001"
    }
  ]
}


import os,sys
import apprcc_rc
from PySide6.QtWidgets import QApplication
from PySide6.QtGui import QIcon
from app_page_core import Store, Param, PageManager
from .core import ThreadManager, Page, Device, Setting, MainWindow
from .config import APP_ICON_PATH, APP_VERSION, APP_TITLE, PING_HOST, IS_DEBUG, tips_ui, tipsBox_ui, loading_icon, small_page_icon, maximize_page_icon
from .utils import setAppStyle

def createApp(SETTING:dict):
  Setting.applySetting({
    'APP_ICON_PATH': APP_ICON_PATH,
    'APP_TITLE': APP_TITLE,
    'APP_VERSION': APP_VERSION,
    'IS_DEBUG': IS_DEBUG,
    'PING_HOST': PING_HOST,
    'tips_ui': tips_ui,
    'tipsBox_ui': tipsBox_ui,
    'loading_icon': loading_icon,
    'small_page_icon': small_page_icon,
    'maximize_page_icon': maximize_page_icon,
  })

  Setting.applySetting(SETTING)

  # 创建应用，添加图标
  app = QApplication(sys.argv)
  app.setWindowIcon(QIcon(APP_ICON_PATH))  # 生成exe时改为绝对路径

  # 创建全局参数对象
  param = Param(filePath=None, default=Device.defaultSystemConfig(version=APP_VERSION))
  user_param = Param(os.path.join(param.get("userPath", ""), "user.json"), {})
  system_param = Param(os.path.join(param.get("systemPath", ""), "system.json"), {})

  # 创建程序主窗口
  main_win = MainWindow(system_param)
  threadManager = ThreadManager()
  pageManager = PageManager()

  # 创建全局变量
  Store({
    'app': app,
    'main_win': main_win,
    'ui': main_win.ui,
    'threadManager': threadManager,
    'pageManager': pageManager,
    'param': param,
    'user_param': user_param,
    'system_param': system_param,
  })

  Root = Setting.getSetting('Root', None)
  if not Root:
    raise Exception("Root is None")
  root:Page = Root()
  root.setup()
  
  # 绑定关闭事件
  main_win.ui.btn_close.clicked.connect(lambda: root.closeApp())

  # 设置样式，必须在创建全局变量之后
  setAppStyle(root)

  n = app.exec()
  try:
      sys.exit(n)
  except SystemExit:
      sys.exit(n)


# 事件总线
class EventBus(object):
  def __init__(self, elements:dict={}):
    self.elements:dict = elements
    
  def getElement(self, id:str):
    return self.elements.get(id, None)
  
  def getConnect(self, id:str, event:str):
    item = self.getElement(id)
    if item:
      connectMap = {
        'clicked': hasattr(item, 'clicked') and hasattr(item.clicked, 'connect') and item.clicked.connect,
        'doubleClicked': hasattr(item, 'doubleClicked') and hasattr(item.doubleClicked, 'connect') and item.doubleClicked.connect,
        'pressed': hasattr(item, 'pressed') and hasattr(item.pressed, 'connect') and item.pressed.connect,
        'released': hasattr(item, 'released') and hasattr(item.released, 'connect') and item.released.connect,
        'hovered': hasattr(item, 'hovered') and hasattr(item.hovered, 'connect') and item.hovered.connect,
        'textChanged': hasattr(item, 'textChanged') and hasattr(item.textChanged, 'connect') and item.textChanged.connect,
        'valueChanged': hasattr(item, 'valueChanged') and hasattr(item.valueChanged, 'connect') and item.valueChanged.connect,
        'currentIndexChanged': hasattr(item, 'currentIndexChanged') and hasattr(item.currentIndexChanged, 'connect') and item.currentIndexChanged.connect,
      }
      return connectMap.get(event, None)
    
  def register(self, id:str, event:str, callback):
    connect = self.getConnect(id, event)
    if connect:
      connect(lambda *args: callback(self, *args))
    
  def clear(self):
    keys = list(self.elements.keys())
    for key in keys:
      del self.elements[key]
    
    self.elements = {}


import setuptools

package_name = "app-page"
version = '0.0.13'
long_description = open("README.md", encoding="utf-8").read()

setuptools.setup(
    name=package_name,
    version=version,
    author="xiaohuicat",  # 作者名称
    author_email="1258702350@qq.com", # 作者邮箱
    description="python page application framework", # 库描述
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/xiaohuicat/python-app-page", # 库的官方地址
    license="MIT",
    packages=setuptools.find_packages(),
    package_data={'app_page': ['*', 'assets/*', 'assets/*/*']},
    install_requires=[
        "app-page-core",
        "nanoid",
        "PySide6",
        "pywin32",
        "numpy",
        "openpyxl",
        "paho-mqtt==1.6.1",
        "pillow"
    ],
    zip_safe=False,
)